searchNodes=[{"ref":"EctoFilter.html","title":"EctoFilter","type":"behaviour","doc":"Aims in building database queries using data as filtering conditions. Filtering conditions Filtering conditions are represented as tuples consisting of field name, filtering rule and value to filter by. For example, {:first_name, :equal, &quot;Bob&quot;} will select all records with the &quot;Bob&quot; value of the :first_name field. You can use multiple conditions, in that case they will be combined in query. Also, it is possible to build queries based on field values of associated models. This can be achieved by condition nesting: {:organization, nil, [{:name, :like, &quot;acme&quot;}]} will select only records with the :name field of their associated as the :organization entity matching with the &quot;acme&quot; value. Operators Every filtering condition is handled by the corresponding operator — a function which takes the Ecto query and the filtering condition and returns the query with the filtering condition applied. Usually operators are defined as apply/4 callback implementations in the filter module. EctoFilter includes default operators for field value comparison, array inclusion checking, matching values with pattern and filtering by values of the associated entities&#39; fields. Also, operators for dealing with PostgreSQL JSON/JSONB fields are available. Extending It is possible to implement custom filtering logic by defining custom filter modules. See examples below for more information. Examples: Basic filtering iex&gt; Repo.insert!(%User{first_name: &quot;Bob&quot;}) iex&gt; Repo.insert!(%User{first_name: &quot;Alice&quot;}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 2 iex&gt; Repo.insert!(%User{email: &quot;bob@example.com&quot;}) iex&gt; Repo.insert!(%User{email: &quot;alice@example.com&quot;}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:email, :equal, &quot;alice@example.com&quot;}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).email &quot;alice@example.com&quot; Building custom filter iex&gt; defmodule CustomFilter do ...&gt; use EctoFilter ...&gt; ...&gt; def apply(query, {:name, :full_text_search, value}, _, User) do ...&gt; where( ...&gt; query, ...&gt; [..., u], ...&gt; fragment(&quot;to_tsvector(concat_ws(&#39; &#39;, ?, ?)) @@ plainto_tsquery(?)&quot;, u.first_name, u.last_name, ^value) ...&gt; ) ...&gt; end ...&gt; ...&gt; def apply(query, condition, type, context), do: super(query, condition, type, context) ...&gt; end iex&gt; Repo.insert!(%User{first_name: &quot;Bob&quot;, last_name: &quot;Doe&quot;}) iex&gt; Repo.insert!(%User{first_name: &quot;Alice&quot;, last_name: &quot;Roe&quot;}) iex&gt; result = ...&gt; User ...&gt; |&gt; CustomFilter.filter([{:name, :full_text_search, &quot;alice roe&quot;}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).first_name &quot;Alice&quot;"},{"ref":"EctoFilter.html#c:apply/4","title":"EctoFilter.apply/4","type":"callback","doc":""},{"ref":"EctoFilter.html#filter/2","title":"EctoFilter.filter/2","type":"function","doc":"Adds filtering conditions to the query."},{"ref":"EctoFilter.html#introspect/2","title":"EctoFilter.introspect/2","type":"function","doc":""},{"ref":"EctoFilter.html#t:condition/0","title":"EctoFilter.condition/0","type":"type","doc":""},{"ref":"EctoFilter.html#t:field_type/0","title":"EctoFilter.field_type/0","type":"type","doc":""},{"ref":"EctoFilter.Default.html","title":"EctoFilter.Default","type":"module","doc":""},{"ref":"EctoFilter.Default.html#apply/4","title":"EctoFilter.Default.apply/4","type":"function","doc":"Callback implementation for EctoFilter.apply/4."},{"ref":"EctoFilter.Default.html#filter/3","title":"EctoFilter.Default.filter/3","type":"function","doc":""},{"ref":"EctoFilter.Operators.Association.html","title":"EctoFilter.Operators.Association","type":"behaviour","doc":"Association operators. Examples: With &quot;one&quot; cardinality iex&gt; acme_org = Repo.insert!(%Organization{name: &quot;Acme&quot;}) iex&gt; globex_org = Repo.insert!(%Organization{name: &quot;Globex&quot;}) iex&gt; Repo.insert!(%User{first_name: &quot;Bob&quot;, organization: acme_org}) iex&gt; Repo.insert!(%User{first_name: &quot;Alice&quot;, organization: globex_org}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:organization, nil, [{:name, :like, &quot;acme&quot;}]}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).first_name &quot;Bob&quot; With &quot;many&quot; cardinality iex&gt; bob = Repo.insert!(%User{first_name: &quot;Bob&quot;}) iex&gt; alice = Repo.insert!(%User{first_name: &quot;Alice&quot;}) iex&gt; Repo.insert!(%Post{title: &quot;Post about weather&quot;, author: bob}) iex&gt; Repo.insert!(%Post{title: &quot;Post about sports&quot;, author: alice}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:posts, nil, [{:title, :like, &quot;sports&quot;}]}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).first_name &quot;Alice&quot; With &quot;many&quot; cardinality through another association iex&gt; weather_post = Repo.insert!(%Post{title: &quot;Post about weather&quot;}) iex&gt; sports_post = Repo.insert!(%Post{title: &quot;Post about sports&quot;}) iex&gt; bob = Repo.insert!(%User{first_name: &quot;Bob&quot;}) iex&gt; alice = Repo.insert!(%User{first_name: &quot;Alice&quot;}) iex&gt; Repo.insert!(%Comment{body: &quot;Lorem ipsum&quot;, author: bob, post: weather_post}) iex&gt; Repo.insert!(%Comment{body: &quot;Dolor sit amet&quot;, author: alice, post: sports_post}) iex&gt; result = ...&gt; Post ...&gt; |&gt; EctoFilter.filter([{:comments_authors, nil, [{:first_name, :like, &quot;bob&quot;}]}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).title &quot;Post about weather&quot;"},{"ref":"EctoFilter.Operators.Association.html#c:apply/4","title":"EctoFilter.Operators.Association.apply/4","type":"callback","doc":""},{"ref":"EctoFilter.Operators.Association.html#t:condition/0","title":"EctoFilter.Operators.Association.condition/0","type":"type","doc":""},{"ref":"EctoFilter.Operators.Comparison.html","title":"EctoFilter.Operators.Comparison","type":"behaviour","doc":"Comparison operators. Examples: With &quot;equal&quot; condition iex&gt; Repo.insert!(%User{email: &quot;bob@example.com&quot;}) iex&gt; Repo.insert!(%User{email: &quot;alice@example.com&quot;}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:email, :equal, &quot;alice@example.com&quot;}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).email &quot;alice@example.com&quot; Comparison with nil are supported: iex&gt; Repo.insert!(%User{email: &quot;bob@example.com&quot;}) iex&gt; Repo.insert!(%User{email: nil}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:email, :equal, nil}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).email nil With &quot;not equal&quot; condition iex&gt; Repo.insert!(%User{email: &quot;bob@example.com&quot;}) iex&gt; Repo.insert!(%User{email: &quot;alice@example.com&quot;}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:email, :not_equal, &quot;alice@example.com&quot;}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).email &quot;bob@example.com&quot; Comparison with nil are also supported: iex&gt; Repo.insert!(%User{email: &quot;bob@example.com&quot;}) iex&gt; Repo.insert!(%User{email: nil}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:email, :not_equal, nil}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).email &quot;bob@example.com&quot; With &quot;less than or equal&quot; condition iex&gt; Repo.insert!(%User{age: 17}) iex&gt; Repo.insert!(%User{age: 18}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:age, :less_than_or_equal, 17}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).age 17 With &quot;greater than or equal&quot; condition iex&gt; Repo.insert!(%User{age: 17}) iex&gt; Repo.insert!(%User{age: 18}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:age, :greater_than_or_equal, 18}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).age 18 With &quot;less than&quot; condition iex&gt; Repo.insert!(%User{age: 17}) iex&gt; Repo.insert!(%User{age: 21}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:age, :less_than, 20}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).age 17 With &quot;greater than&quot; condition iex&gt; Repo.insert!(%User{age: 17}) iex&gt; Repo.insert!(%User{age: 21}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:age, :greater_than, 18}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).age 21"},{"ref":"EctoFilter.Operators.Comparison.html#c:apply/4","title":"EctoFilter.Operators.Comparison.apply/4","type":"callback","doc":""},{"ref":"EctoFilter.Operators.Comparison.html#t:condition/0","title":"EctoFilter.Operators.Comparison.condition/0","type":"type","doc":""},{"ref":"EctoFilter.Operators.Comparison.html#t:rule/0","title":"EctoFilter.Operators.Comparison.rule/0","type":"type","doc":""},{"ref":"EctoFilter.Operators.Inclusion.html","title":"EctoFilter.Operators.Inclusion","type":"behaviour","doc":"Inclusion operators. Examples: iex&gt; Repo.insert!(%Post{category: &quot;News&quot;}) iex&gt; Repo.insert!(%Post{category: &quot;Sports&quot;}) iex&gt; result = ...&gt; Post ...&gt; |&gt; EctoFilter.filter([{:category, :in, [&quot;News&quot;, &quot;Weather&quot;]}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).category &quot;News&quot; iex&gt; Repo.insert!(%Post{tags: [&quot;News&quot;, &quot;Sports&quot;]}) iex&gt; Repo.insert!(%Post{tags: [&quot;Weather&quot;]}) iex&gt; result = ...&gt; Post ...&gt; |&gt; EctoFilter.filter([{:tags, :contains, &quot;Sports&quot;}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).tags [&quot;News&quot;, &quot;Sports&quot;]"},{"ref":"EctoFilter.Operators.Inclusion.html#c:apply/4","title":"EctoFilter.Operators.Inclusion.apply/4","type":"callback","doc":""},{"ref":"EctoFilter.Operators.Inclusion.html#t:condition/0","title":"EctoFilter.Operators.Inclusion.condition/0","type":"type","doc":""},{"ref":"EctoFilter.Operators.Inclusion.html#t:rule/0","title":"EctoFilter.Operators.Inclusion.rule/0","type":"type","doc":""},{"ref":"EctoFilter.Operators.JSON.html","title":"EctoFilter.Operators.JSON","type":"behaviour","doc":"Operators on PostgreSQL JSON/JSONB fields. These operators are not included in EctoFilter by default and must be used manually: defmodule JSONFilter do use EctoFilter use EctoFilter.Operators.JSON end Examples: With arrays iex&gt; Repo.insert(%User{first_name: &quot;Bob&quot;, interests: [&quot;Art&quot;, &quot;Books&quot;]}) iex&gt; Repo.insert(%User{first_name: &quot;Alice&quot;, interests: [&quot;Books&quot;, &quot;Comics&quot;]}) iex&gt; result = ...&gt; User ...&gt; |&gt; JSONFilter.filter([{:interests, :contains, &quot;Art&quot;}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).first_name &quot;Bob&quot; With maps iex&gt; Repo.insert(%User{first_name: &quot;Bob&quot;, settings: %{send_newsletter: false}}) iex&gt; Repo.insert(%User{first_name: &quot;Alice&quot;, settings: %{send_newsletter: true}}) iex&gt; result = ...&gt; User ...&gt; |&gt; JSONFilter.filter([{:settings, nil, [{:send_newsletter, :equal, true}]}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).first_name &quot;Alice&quot; With nested maps iex&gt; Repo.insert(%User{first_name: &quot;Bob&quot;, settings: %{send_newsletter: %{weekly: true, monthly: false}}}) iex&gt; Repo.insert(%User{first_name: &quot;Alice&quot;, settings: %{send_newsletter: %{weekly: false, monthly: true}}}) iex&gt; result = ...&gt; User ...&gt; |&gt; JSONFilter.filter([{:settings, nil, [{:send_newsletter, nil, [{:weekly, :equal, true}]}]}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).first_name &quot;Bob&quot; With arrays of maps iex&gt; Repo.insert(%User{first_name: &quot;Bob&quot;, addresses: [%{city: &quot;Kyiv&quot;}, %{city: &quot;Berlin&quot;}]}) iex&gt; Repo.insert(%User{first_name: &quot;Alice&quot;, addresses: [%{city: &quot;Chicago&quot;}]}) iex&gt; result = ...&gt; User ...&gt; |&gt; JSONFilter.filter([{:addresses, nil, [{:city, :equal, &quot;Kyiv&quot;}]}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).first_name &quot;Bob&quot;"},{"ref":"EctoFilter.Operators.JSON.html#c:apply/4","title":"EctoFilter.Operators.JSON.apply/4","type":"callback","doc":""},{"ref":"EctoFilter.Operators.JSON.html#t:condition/0","title":"EctoFilter.Operators.JSON.condition/0","type":"type","doc":""},{"ref":"EctoFilter.Operators.JSON.html#t:field_type/0","title":"EctoFilter.Operators.JSON.field_type/0","type":"type","doc":""},{"ref":"EctoFilter.Operators.PatternMatching.html","title":"EctoFilter.Operators.PatternMatching","type":"behaviour","doc":"Pattern matching operators. Examples: With &quot;LIKE&quot; condition iex&gt; Repo.insert!(%User{first_name: &quot;Bob&quot;}) iex&gt; Repo.insert!(%User{first_name: &quot;Alice&quot;}) iex&gt; result = ...&gt; User ...&gt; |&gt; EctoFilter.filter([{:first_name, :like, &quot;al&quot;}]) ...&gt; |&gt; Repo.all() iex&gt; length(result) 1 iex&gt; hd(result).first_name &quot;Alice&quot;"},{"ref":"EctoFilter.Operators.PatternMatching.html#c:apply/4","title":"EctoFilter.Operators.PatternMatching.apply/4","type":"callback","doc":""},{"ref":"EctoFilter.Operators.PatternMatching.html#t:condition/0","title":"EctoFilter.Operators.PatternMatching.condition/0","type":"type","doc":""},{"ref":"EctoFilter.Operators.PatternMatching.html#t:rule/0","title":"EctoFilter.Operators.PatternMatching.rule/0","type":"type","doc":""}]